/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 1.0.2, packaged on February2008.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#ifndef OPENGLVIEW_H_
#define OPENGLVIEW_H_

#include <QGLWidget>
#include <QFile>
#include <glc_viewport.h>
#include <glc_collection.h>
#include <glc_light.h>
#include <glc_factory.h>

// The State of OpenGL view
enum ViewState_enum
{
	V_NORMAL= 1, V_ORBITING, V_PANNING, V_ZOOMING
};

enum ViewEnterState_enum
{
	VE_NORMAL= 10, VE_PANNING, VE_ORBITING, VE_POINTING, VE_ZOOMING
};

//////////////////////////////////////////////////////////////////////
//! \class OpenglView
/*! \brief OpenglView : The OpenGl view of GLC_Player*/
//////////////////////////////////////////////////////////////////////
class OpenglView : public QGLWidget
{
	Q_OBJECT

//////////////////////////////////////////////////////////////////////
/*! @name Constructor / Destructor */
//@{
//////////////////////////////////////////////////////////////////////
public:
	OpenglView(QWidget *);
	virtual ~OpenglView();
//@}

//////////////////////////////////////////////////////////////////////
// Public Interface
//////////////////////////////////////////////////////////////////////
public:
	//! Initialyze OpenGL window
	void initializeGL();
	//! Reframe the Scene
	void reframe(const GLC_BoundingBox&, bool);
	//! Change The enter state of the view
	void changeEnterState(ViewEnterState_enum);
	//! Clear the view
	inline void clear() {m_GeomCollection.clear();}
	//! Add instance in the view
	inline void add(GLC_Instance& instance) {m_GeomCollection.add(instance);}
	//! Get Mode
	inline GLenum getMode() const {return m_Mode;}
	//! Set Mode
	inline void setMode(GLenum mode)
	{
		m_Mode= mode;
		m_GeomCollection.setPolygonModeForAll(GL_FRONT_AND_BACK, mode);
	}
	//! bool isEmpty
	inline bool isEmpty() const {return m_GeomCollection.isEmpty();}
	//! Get the view Camera
	inline GLC_Camera getCamera() const {return *(m_GlView.m_pViewCam);}
	//! Set the view Camera
	void setCamera(const GLC_Camera&);
	//! Set AutoBufferSwap
	inline void setAutoBufferSwap(bool on) {QGLWidget::setAutoBufferSwap(on);}
	//! unselect Instance
	void unselect();
	//! set Info Panel visibility
	inline void setDisplayInfoPanel(bool vis){m_dislayInfoPanel= vis;}
	//! set the snapshoot Mode
	inline void setSnapShootMode(bool mode){m_SnapShootMode= mode;}

public slots:
	// Iso view
	void isoView(bool motion= true);
	// Front View
	void frontView();
	// Back View
	void backView();
	// Left View
	void leftView();
	// Right View
	void rightView();
	// Top View
	void topView();
	// Bottom View
	void bottomView();
	// Zoom in
	void zoomIn();
	// Zoom out
	void zoomOut();

//////////////////////////////////////////////////////////////////////
// Signals
//////////////////////////////////////////////////////////////////////
signals:
	//! Progression of loading file quantum
	void currentQuantum(int);
	void selectInstance(GLC_Instance*);
	void unselectGeom();
	void hideInfoPanel();
	
//////////////////////////////////////////////////////////////////////
// Private services Functions
//////////////////////////////////////////////////////////////////////
private:
	void paintGL();
	void resizeGL(int width, int height);
	//Mouse events
	void mousePressEvent(QMouseEvent *);
	void mouseReleaseEvent(QMouseEvent *);
	void mouseMoveEvent(QMouseEvent *);
	void wheelEvent(QWheelEvent *);
	//! Rotate the camera softely with specified step
	void rotateCamera(const GLC_Camera, const int);
	//! reframe softely with specified step
	void motionReframe(const GLC_Camera, const int);
	//! Update the current fps
	void updateFps(int);
	//! Display info panel
	void displayInfo();
	//! Select
	void select(int, int);

//////////////////////////////////////////////////////////////////////
// Private members
//////////////////////////////////////////////////////////////////////
private:
	GLC_Viewport m_GlView;
	GLC_Collection m_GeomCollection;
	GLC_Light m_Light;
	GLenum m_Mode;
	ViewState_enum m_ViewState;
	ViewEnterState_enum m_ViewEnterState;
	int m_MotionStep;
	// fps calcul
	int m_CurrentFps;
	QFont m_infoFont;
	//! bool in selection mode
	bool m_SelectionMode;
	bool m_dislayInfoPanel;
	bool m_SnapShootMode;
	
};

#endif /*OPENGLVIEW_H_*/
