/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 1.0.2, packaged on February2008.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "glc_player.h"
#include "SettingsDialog.h"
#include "AboutPlayer.h"
#include "EditMeshDialog.h"

glc_player::glc_player(QWidget *parent)
: QMainWindow(parent)
, m_OpenglView(this)
, m_CurrentPath(QDir::homePath())
, m_pProgressBar(new QProgressBar())
, m_pQErrorMessage(NULL)
, m_RecentFilesList()
, m_pRecentFileSeparator(NULL)
, m_CurrentFileName()
, m_OpenFileThread(GLC_Factory::instance(m_OpenglView.context()))
, m_FileLoadingInProgress(false)
, m_ListLoadingInProgress(false)
, m_ContinuListLoading(true)
, m_NumberOfUnloadedFile(0)
, m_NumberOfErrorFile(0)
, m_iconSize()
, m_MakeFirstFileCurrent(true)
, m_DisplayThumbnails(true)
, m_CreateThumbnailWhileLoading(true)
, m_pActionDeleteSelectedItem(new QAction(tr("Remove Model"), this))
, m_pCurrentSelection(NULL)
, m_dislayInfoPanel()
{
	ui.setupUi(this);
	setCentralWidget(&m_OpenglView);
	readSettings();
	createRecentFileActionsArray();
	updateRecentsFiles();
	
	// Display info panel
	m_OpenglView.setDisplayInfoPanel(m_dislayInfoPanel);
	
	// Resize Texture cache limite to 128 Mo
	QGLContext::setTextureCacheLimit(128 * 1024);
	
	// Item list Icons
	ui.loadingList->setIconSize(m_iconSize);

	// Accept drop event
	m_OpenglView.setAcceptDrops(false);
	setAcceptDrops(true);
	
	// Set the current file name
	addToRecentFiles(m_CurrentFileName);
	
	
	// Status bar
	ui.statusbar->addPermanentWidget(m_pProgressBar);
	m_pProgressBar->hide();	
	connect(&m_OpenglView, SIGNAL(currentQuantum(int)), this, SLOT(updateProgressBar(int)), Qt::QueuedConnection);
		
	// QAction Signals and slots connection
	// Signals from the view
	connect(&m_OpenglView, SIGNAL(selectInstance(GLC_Instance*)), this, SLOT(select(GLC_Instance*)));
	connect(&m_OpenglView, SIGNAL(unselectGeom()), this, SLOT(unselect()));
	connect(&m_OpenglView, SIGNAL(hideInfoPanel()), this, SLOT(hideInfoPanel()));
	//Menu File
	connect(ui.action_NewSession, SIGNAL(triggered()), this , SLOT(newSession()));
	connect(ui.action_Open, SIGNAL(triggered()), this, SLOT(open()));
	connect(ui.action_Quit, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));
	// Menu edit
	connect(ui.action_Property, SIGNAL(triggered()), this, SLOT(meshProperty()));
	//Menu Window
	connect(ui.action_SessionManagement, SIGNAL(triggered()), this, SLOT(sessionManagementVisibilityToggle()));
	connect(ui.sessionManagementWindow, SIGNAL(visibilityChanged(bool)), this, SLOT(sessionManagementVisibilityChanged(bool)));
	//Menu View
	connect(ui.action_Reframe, SIGNAL(triggered()), this, SLOT(reframe()));
	connect(ui.action_Select, SIGNAL(triggered()), this, SLOT(selectMode()));
	connect(ui.action_ViewCenter, SIGNAL(triggered()), this, SLOT(viewCenterMode()));
	connect(ui.action_Pan, SIGNAL(triggered()), this, SLOT(panMode()));
	connect(ui.action_Rotate, SIGNAL(triggered()), this, SLOT(rotateMode()));
	connect(ui.action_Zoom, SIGNAL(triggered()), this, SLOT(zoomMode()));
	connect(ui.action_ZoomIn, SIGNAL(triggered()), &m_OpenglView, SLOT(zoomIn()));
	connect(ui.action_ZoomOut, SIGNAL(triggered()), &m_OpenglView, SLOT(zoomOut()));
	connect(ui.action_FullScreen, SIGNAL(triggered()), this, SLOT(fullScreen()));
	
	// predifined view 
	connect(ui.action_IsoView, SIGNAL(triggered()), &m_OpenglView, SLOT(isoView()));
	connect(ui.action_FrontView, SIGNAL(triggered()), &m_OpenglView, SLOT(frontView()));
	connect(ui.action_BackView, SIGNAL(triggered()), &m_OpenglView, SLOT(backView()));
	connect(ui.action_LeftView, SIGNAL(triggered()), &m_OpenglView, SLOT(leftView()));
	connect(ui.action_RightView, SIGNAL(triggered()), &m_OpenglView, SLOT(rightView()));
	connect(ui.action_TopView, SIGNAL(triggered()), &m_OpenglView, SLOT(topView()));
	connect(ui.action_BottomView, SIGNAL(triggered()), &m_OpenglView, SLOT(bottomView()));
	
	// Menu Render
	// Render Mode
	connect(ui.action_RenderPoints, SIGNAL(triggered()), this, SLOT(pointsRenderingMode()));
	connect(ui.action_RenderWireframe, SIGNAL(triggered()), this, SLOT(wireframeRenderingMode()));
	connect(ui.action_RenderShading, SIGNAL(triggered()), this, SLOT(shadingRenderingMode()));
	
	// Menu Tools
	connect(ui.action_Settings, SIGNAL(triggered()), this, SLOT(showSettings()));
	connect(ui.action_SnapShot, SIGNAL(triggered()), this, SLOT(takeSnapShot()));
	
	// Menu Help
	connect(ui.action_About, SIGNAL(triggered()), this, SLOT(aboutPlayer()));
	connect(ui.action_Help, SIGNAL(triggered()), this, SLOT(help()));
	
	// Open File thread
	connect(&m_OpenFileThread, SIGNAL(currentQuantum(int)), this, SLOT(updateProgressBar(int)), Qt::QueuedConnection);
	connect(&m_OpenFileThread, SIGNAL(finished()), this, SLOT(fileOpened()), Qt::QueuedConnection);
	connect(&m_OpenFileThread, SIGNAL(loadError()), this, SLOT(loadFileFailed()), Qt::QueuedConnection);
	
	// Item list dock window
	connect(ui.displayItemCheckBox, SIGNAL(stateChanged(int)), this , SLOT(manageItemInformationDisplay(int)));
	connect(ui.refreshThumbnails, SIGNAL(clicked()), this , SLOT(refreshIconInItemList()));
	connect(ui.removeUnloadItemButton, SIGNAL(clicked()), this , SLOT(removeUnloadFileItem()));
	connect(ui.removeOnErrorItem, SIGNAL(clicked()), this , SLOT(removeItemOnError()));
	connect(ui.startLoadingButton, SIGNAL(clicked()), this , SLOT(startLoadingButton()));
	connect(ui.stopLoadingButton, SIGNAL(clicked()), this , SLOT(stopLoadingButton()));
	
	// List Widget
	connect(ui.loadingList, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *))
			, this , SLOT(currentFileItemChanged(QListWidgetItem *, QListWidgetItem *)));
	connect(ui.loadingList, SIGNAL(itemClicked(QListWidgetItem *)), this , SLOT(fileItemClicked(QListWidgetItem *)));
	
	
	//Contextual menu Creation
	ui.loadingList->addAction(m_pActionDeleteSelectedItem);
	m_pActionDeleteSelectedItem->setShortcut(Qt::Key_Delete);
	connect(m_pActionDeleteSelectedItem, SIGNAL(triggered()), this, SLOT(deleteCurrentItem()));
	m_pActionDeleteSelectedItem->setEnabled(false);
	
	// Parse arguments command line
	QStringList args= QCoreApplication::arguments ();
	if (args.size() > 1)
	{
		addItems(QStringList(QFileInfo(args[1]).filePath()));
		startLoading();
	}
	else
	{
		updateUIState();
	}
}

glc_player::~glc_player()
{
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	// delete the progress bar
	if (NULL != m_pProgressBar)
	{
		delete m_pProgressBar;
	}
	
	// delete error message dialog
	if (NULL != m_pQErrorMessage)
	{
		delete m_pQErrorMessage;
	}
	ui.loadingList->clear();
	m_FileEntryHash.clear();
	QApplication::restoreOverrideCursor();
}
//////////////////////////////////////////////////////////////////////
// Virtual protected function
//////////////////////////////////////////////////////////////////////
// Close event handler
void glc_player::closeEvent(QCloseEvent* pEvent)
{
	int ret= QMessageBox::Yes;
	//! Display confirmation msgBox only if there session is not empty
	if (ui.loadingList->count() > 0)
	{
		ret= QMessageBox::question(this, QCoreApplication::applicationName(),
				QString(tr("Quits ")) +  QCoreApplication::applicationName() + QString("?"), QMessageBox::Yes | QMessageBox::No);
	}
	if (ret == QMessageBox::Yes)
	{
		ui.loadingList->blockSignals(true);
		writeSettings();
		pEvent->accept();
		QCoreApplication::quit();
	}
	else
	{
		pEvent->ignore();
	}
}
//////////////////////////////////////////////////////////////////////
// public Slots
//////////////////////////////////////////////////////////////////////
// an instance as been selected
void glc_player::select(GLC_Instance* pInstance)
{
	m_pCurrentSelection= pInstance;
	ui.action_Property->setEnabled(true);
}

// an instance as been unselected
void glc_player::unselect()
{
	m_pCurrentSelection= NULL;
	ui.action_Property->setEnabled(false);
}

// Info panel not supported
void glc_player::hideInfoPanel()
{
	m_dislayInfoPanel= false;
	writeSettings();
	QCoreApplication::quit();	
}

//////////////////////////////////////////////////////////////////////
// private Slots
//////////////////////////////////////////////////////////////////////
// update the value of the progress bar
void glc_player::updateProgressBar(int value)
{
	if (value < 100)
	{
		ui.statusbar->showMessage(tr("Loading in Progress Please Wait"));
		m_pProgressBar->setValue(value);
	}
	else
	{
		ui.statusbar->showMessage(tr("File Loaded"));
		m_pProgressBar->hide();
	}
}


//////////////////////////////////////////////////////////////////////
// private Slots
//////////////////////////////////////////////////////////////////////
// Open An existing file
void glc_player::open()
{
	QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Select File(s) to Open and Add in session")
	, m_CurrentPath, "Alias OBJ (*.obj)");
	if (!fileNames.isEmpty())
	{
		if (m_NumberOfUnloadedFile == 0) m_MakeFirstFileCurrent= true;
		addItems(fileNames);
		startLoading();
	}
}

// Open a Recent File
void glc_player::openRecentFile()
{
	QAction* action= qobject_cast<QAction*>(sender());
	if (action)
	{
		if (m_NumberOfUnloadedFile == 0) m_MakeFirstFileCurrent= true;
		addItems(QStringList(action->data().toString()));
		startLoading();
	}
}

// View and edit mesh property
void glc_player::meshProperty()
{
	EditMeshDialog editMeshDialog(m_OpenglView.context(), m_pCurrentSelection, this);
	editMeshDialog.exec();	
}

// Reframe the view
void glc_player::reframe()
{
	GLC_BoundingBox emptyBoundingBox;
	m_OpenglView.reframe(emptyBoundingBox, true);
	
}
// Select mode
void glc_player::selectMode()
{
	if (!ui.action_Select->isChecked())
	{
		ui.action_Select->setChecked(true);
	}
	ui.action_ViewCenter->setChecked(false);
	ui.action_Pan->setChecked(false);
	ui.action_Rotate->setChecked(false);
	ui.action_Zoom->setChecked(false);
	m_OpenglView.changeEnterState(VE_NORMAL);
}

// define view center
void glc_player::viewCenterMode()
{
	if (!ui.action_ViewCenter->isChecked())
	{
		ui.action_ViewCenter->setChecked(true);
	}
	ui.action_Select->setChecked(false);
	ui.action_Pan->setChecked(false);
	ui.action_Rotate->setChecked(false);
	ui.action_Zoom->setChecked(false);
	m_OpenglView.changeEnterState(VE_POINTING);	
}

// Pan the view
void glc_player::panMode()
{
	if (!ui.action_Pan->isChecked())
	{
		ui.action_Pan->setChecked(true);
	}
	ui.action_Select->setChecked(false);
	ui.action_ViewCenter->setChecked(false);
	ui.action_Rotate->setChecked(false);
	ui.action_Zoom->setChecked(false);
	m_OpenglView.changeEnterState(VE_PANNING);	
}

// Rotate the view
void glc_player::rotateMode()
{
	if (!ui.action_Rotate->isChecked())
	{
		ui.action_Rotate->setChecked(true);
	}	
	ui.action_Select->setChecked(false);
	ui.action_ViewCenter->setChecked(false);
	ui.action_Pan->setChecked(false);
	ui.action_Zoom->setChecked(false);
	m_OpenglView.changeEnterState(VE_ORBITING);	
}

// Zoom the view mode
void glc_player::zoomMode()
{
	if (!ui.action_Zoom->isChecked())
	{
		ui.action_Zoom->setChecked(true);
	}	
	ui.action_Select->setChecked(false);
	ui.action_ViewCenter->setChecked(false);
	ui.action_Pan->setChecked(false);
	ui.action_Rotate->setChecked(false);
	m_OpenglView.changeEnterState(VE_ZOOMING);	
}

// Show application settings dialog
void glc_player::showSettings()
{
	SettingsDialog settingsDialog(m_DisplayThumbnails, m_CreateThumbnailWhileLoading, m_iconSize, m_dislayInfoPanel, this);
	if (settingsDialog.exec() == QDialog::Accepted)
	{
		bool update= m_DisplayThumbnails != settingsDialog.thumbnailHaveToBeDisplay();
		update= update || (m_CreateThumbnailWhileLoading != settingsDialog.thumbnailHaveToBeCreated());
		update= update || (m_iconSize != settingsDialog.getThumbnailSize());
		if (update)
		{
			m_DisplayThumbnails= settingsDialog.thumbnailHaveToBeDisplay();
			// Disable Refresh thumbnail button
			ui.refreshThumbnails->setEnabled(m_DisplayThumbnails & (ui.loadingList->count() != 0));	
			m_CreateThumbnailWhileLoading= settingsDialog.thumbnailHaveToBeCreated();		
			m_iconSize= settingsDialog.getThumbnailSize();
			settingsDialog.hide();
			ui.loadingList->setIconSize(m_iconSize);
			refreshIconInItemList();
		}
		if (m_dislayInfoPanel != settingsDialog.infoHaveToBeShown())
		{
			m_dislayInfoPanel= settingsDialog.infoHaveToBeShown();
			m_OpenglView.setDisplayInfoPanel(m_dislayInfoPanel);
		}
	}
	
}
// A file as been Opened
void glc_player::fileOpened()
{
	GLC_Instance instance(m_OpenFileThread.getInstance());
	const QString fileName(m_OpenFileThread.getFileName());
	// Decrement the number of unload file
	--m_NumberOfUnloadedFile;
	// Check if the an instance as been succesffuly built
	if (!instance.isNull())
	{
		m_FileEntryHash[fileName].setInstance(instance);
		m_FileEntryHash[fileName].setLoadingStatus(false);
		// Update the foreground of the item Opened file
		int curItem= 0;
		while (ui.loadingList->item(curItem)->data(Qt::UserRole).toString() != fileName)
		{
			++curItem;
		}
		ui.loadingList->item(curItem)->setForeground(QBrush(Qt::black));
		// Modify model default material
		GLC_Material* pMaterial= new GLC_Material(Qt::black);
		pMaterial->setDiffuseColor(QColor::fromRgbF(0.97, 0.97, 1.00));
		pMaterial->setShininess(18.0);
		instance.getGeometry()->setMaterial(pMaterial);
		
		// Test if there is something in the view or if there is unload files
		if (m_MakeFirstFileCurrent)
		{
			//Update window Title
			setWindowTitle(QCoreApplication::applicationName() + QString(" [") + QFileInfo(fileName).fileName() + QString("]"));
			// Set loading file as current item
			if (ui.loadingList->item(curItem) != ui.loadingList->currentItem())
			{
				ui.loadingList->setCurrentItem(ui.loadingList->item(curItem));
			}
			else	// force the display of loaded file
			{
				currentFileItemChanged(ui.loadingList->currentItem(), NULL);
			}
		}
		// Create the icon of the loaded file
		if (m_DisplayThumbnails && m_CreateThumbnailWhileLoading)
		{
			computeIconInBackBuffer(curItem);
		}
		addToRecentFiles(fileName);
		m_FileLoadingInProgress= false;
		m_MakeFirstFileCurrent= false;
	}
	updateUIState();
	// If there is a other file item, load it
	startLoading();
}
// load of File failed
void glc_player::loadFileFailed()
{
	ui.statusbar->showMessage(tr("File Not Loaded"));
	m_pProgressBar->hide();
	
	// Update the foreground of the item Opened file
	const QString fileName(m_OpenFileThread.getFileName());
	int curItem= 0;
	while (ui.loadingList->item(curItem)->data(Qt::UserRole).toString() != fileName)
	{
		++curItem;
	}
	ui.loadingList->item(curItem)->setForeground(QBrush(Qt::red));
	// Update the entry
	m_FileEntryHash[fileName].setLoadingStatus(false);
	m_FileEntryHash[fileName].setError(m_OpenFileThread.getErrorMsg());
	m_FileLoadingInProgress= false;
	// Test if the item on error is the first
	if (m_NumberOfErrorFile == 0)
		ui.removeOnErrorItem->setEnabled(true);
	// increment the number of load failure
	++m_NumberOfErrorFile;
}
// Current file Item Changed
void glc_player::currentFileItemChanged(QListWidgetItem * current, QListWidgetItem * previous )
{
	if ((previous != NULL) && (current != NULL))
	{
		if (NULL != m_pCurrentSelection)
		{
			unselect();
			// view feedback
			m_OpenglView.unselect();
		}
		unselect();
		QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(previous->data(Qt::UserRole).toString());
		if (iEntry.value().isLoaded() && (!(iEntry.value().getCamera() == m_OpenglView.getCamera())
											|| (iEntry.value().getIconMode() != m_OpenglView.getMode())))
		{
			iEntry.value().setCamera(m_OpenglView.getCamera());
			iEntry.value().setIconMode(m_OpenglView.getMode());
			if (m_DisplayThumbnails)
			{
				computeIconInBackBuffer(ui.loadingList->row(previous));
				/*
				QImage snapShoot= m_OpenglView.grabFrameBuffer();
				previous->setIcon(QPixmap::fromImage(snapShoot).scaled(m_iconSize, Qt::IgnoreAspectRatio));
				*/
			}
		}
	}
	if (current != NULL)
	{
		setCurrentFileItem(current->data(Qt::UserRole).toString());
	}
	else
	{
		m_OpenglView.clear();
		m_OpenglView.updateGL();		
	}
	// Update UI
	if (m_OpenglView.isEmpty() && ui.action_IsoView->isEnabled())
	{
		ui.action_IsoView->setEnabled(false);
		ui.action_FrontView->setEnabled(false);
		ui.action_BackView->setEnabled(false);
		ui.action_LeftView->setEnabled(false);
		ui.action_RightView->setEnabled(false);
		ui.action_TopView->setEnabled(false);
		ui.action_BottomView->setEnabled(false);
		ui.action_Reframe->setEnabled(false);
		ui.action_ZoomIn->setEnabled(false);
		ui.action_ZoomOut->setEnabled(false);
		ui.action_RenderPoints->setEnabled(false);
		ui.action_RenderWireframe->setEnabled(false);
		ui.action_RenderShading->setEnabled(false);
		ui.action_SnapShot->setEnabled(false);
	}
	else if (!m_OpenglView.isEmpty() && !ui.action_IsoView->isEnabled())
	{
		ui.action_IsoView->setEnabled(true);
		ui.action_FrontView->setEnabled(true);
		ui.action_BackView->setEnabled(true);
		ui.action_LeftView->setEnabled(true);
		ui.action_RightView->setEnabled(true);
		ui.action_TopView->setEnabled(true);
		ui.action_BottomView->setEnabled(true);
		ui.action_Reframe->setEnabled(true);
		ui.action_ZoomIn->setEnabled(true);
		ui.action_ZoomOut->setEnabled(true);
		ui.action_RenderPoints->setEnabled(true);
		ui.action_RenderWireframe->setEnabled(true);
		ui.action_RenderShading->setEnabled(true);
		ui.action_SnapShot->setEnabled(true);
	}
}
// File Item as been clicked
void glc_player::fileItemClicked(QListWidgetItem *pClickedItem)
{
	if (pClickedItem == ui.loadingList->currentItem())
	{
		ui.statusbar->showMessage(pClickedItem->data(Qt::UserRole).toString());
	}
}

// Remove all file item from the current session
void glc_player::newSession()
{
	if (!m_FileLoadingInProgress)
	{
		int ret= QMessageBox::question(this, tr("New Session Confirmation"),
								 tr("Remove All Model from the current session"), QMessageBox::Yes | QMessageBox::No);
		if (ret == QMessageBox::Yes)
		{
			QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
			//ui.loadingList->blockSignals(true);	
			ui.loadingList->clear();
			//ui.loadingList->blockSignals(false);
			m_FileEntryHash.clear();
			m_OpenglView.clear();
			m_OpenglView.updateGL();
			
			// Update number of loaded file and error
			m_NumberOfErrorFile= 0;
			m_NumberOfUnloadedFile= 0;
			// Update Continu list loading flag
			m_ContinuListLoading= true;
			// Upadet UI
			updateUIState();
			QApplication::restoreOverrideCursor();
		}
	}
}
// Remove unload item
void glc_player::removeUnloadFileItem()
{
	int currentRow= ui.loadingList->currentRow();
	for (int i= 0; i < m_NumberOfUnloadedFile; ++i)
	{
		int row= ui.loadingList->count() - 1;
		m_FileEntryHash.remove(ui.loadingList->item(row)->data(Qt::UserRole).toString());
		delete ui.loadingList->takeItem(ui.loadingList->count() - 1);
	}
	m_NumberOfUnloadedFile= 0;
	// Update Continu list loading flag
	m_ContinuListLoading= true;
	updateUIState();
	if (currentRow < ui.loadingList->count())
	{
		ui.loadingList->setCurrentRow(currentRow);
	}
	else
	{
		
		ui.loadingList->setCurrentRow(0);
	}
	
	
}
// Remove item on error
void glc_player::removeItemOnError()
{
	QString key;
	QHash<QString, FileEntry>::iterator iEntry;
	int i= 0;
	while ((i < ui.loadingList->count()) && (m_NumberOfErrorFile > 0))
	{
		key= ui.loadingList->item(i)->data(Qt::UserRole).toString();
		iEntry= m_FileEntryHash.find(key);
		if (iEntry.value().errorOccurWhileLoading())
		{
			m_FileEntryHash.remove(key);
			delete ui.loadingList->takeItem(i);
			--m_NumberOfErrorFile;
		}
		else
		{
			++i;
		}
	}
	updateUIState();
}

// Start Loading
void glc_player::startLoadingButton()
{
	if (ui.startLoadingButton->isChecked())
	{
		ui.stopLoadingButton->setChecked(false);
		m_ContinuListLoading= true;
		startLoading();
	}
	else
	{
		ui.startLoadingButton->setChecked(true);
	}
	
}
// Stop Loading
void glc_player::stopLoadingButton()
{
	if (ui.stopLoadingButton->isChecked())
	{
		ui.startLoadingButton->setChecked(false);
		m_ContinuListLoading= false;
		QString loadingFileItemName= m_OpenFileThread.getFileName();
		QString msg(QString(tr("Loading will be interrupt after the loading of : ")) + loadingFileItemName);
		QMessageBox::information(this, tr("Loading interruption"), msg);		
	}
	else
	{
		ui.stopLoadingButton->setChecked(true);
	}
}

// Take a snapshoot
void glc_player::takeSnapShot()
{
	if (!m_OpenglView.isEmpty())
	{
		QFileInfo fileInfo(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		QString defaultFileName(fileInfo.absolutePath() + QDir::separator() + fileInfo.baseName() + QString(".png"));		
		QString fileName= QFileDialog::getSaveFileName(this, tr("Save snapShoot"), defaultFileName , tr("Images (*.png *.jpg)"));
		if (!fileName.isEmpty())
		{
			m_OpenglView.setSnapShootMode(true);
			m_OpenglView.setAutoBufferSwap(false);
			m_OpenglView.updateGL();
			QImage snapShoot= m_OpenglView.grabFrameBuffer();
			m_OpenglView.setSnapShootMode(false);
			m_OpenglView.updateGL();
			m_OpenglView.setAutoBufferSwap(true);
			snapShoot.save(fileName);
		}
	}
}
// Delete the currentItem
void glc_player::deleteCurrentItem()
{
	if (NULL != ui.loadingList->currentItem())
	{
		int currentRow= ui.loadingList->currentRow();
		QString fileName(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		delete ui.loadingList->takeItem(currentRow);
		// Take the entry from hash table
		FileEntry entry= m_FileEntryHash.take(fileName);
		// Test entry status
		if (entry.errorOccurWhileLoading())
		{
			--m_NumberOfErrorFile;
		}
		else if (entry.isReadyToLoad())
		{
			if (--m_NumberOfUnloadedFile == 0)
			{
				// Update Continu list loading flag
				m_ContinuListLoading= true;
			}
		}
		updateUIState();
	}
}

// Manage current item information displaying
void glc_player::manageItemInformationDisplay(int state)
{
	if (state == Qt::Unchecked)
	{
		ui.currentItemGroup->hide();
	}
	else
	{
		ui.currentItemGroup->show();
	}
}
// Session Management visibility toggle
void glc_player::sessionManagementVisibilityToggle()
{
	ui.sessionManagementWindow->blockSignals(true);
	if (ui.action_SessionManagement->isChecked())
	{
		ui.sessionManagementWindow->show();
	}
	else
	{
		ui.sessionManagementWindow->hide();
	}
	ui.sessionManagementWindow->blockSignals(false);
}
// The session management visibility change
void glc_player::sessionManagementVisibilityChanged(bool isVisible)
{
	ui.action_SessionManagement->setChecked(isVisible);
}

// Go to Point Rendering mode
void glc_player::pointsRenderingMode()
{
	if (!ui.action_RenderPoints->isChecked())
	{
		ui.action_RenderPoints->setChecked(true);
	}
	else
	{
		ui.action_RenderWireframe->setChecked(false);
		ui.action_RenderShading->setChecked(false);
		m_OpenglView.setMode(GL_POINT);
		m_OpenglView.updateGL();
	}
}
// Go to wireframe Rendering mode
void glc_player::wireframeRenderingMode()
{
	if (!ui.action_RenderWireframe->isChecked())
	{
		ui.action_RenderWireframe->setChecked(true);
	}
	else
	{
		ui.action_RenderPoints->setChecked(false);
		ui.action_RenderShading->setChecked(false);
		m_OpenglView.setMode(GL_LINE);
		m_OpenglView.updateGL();
	}
}
// Go to shading Rendering Mode
void glc_player::shadingRenderingMode()
{
	if (!ui.action_RenderShading->isChecked())
	{
		ui.action_RenderShading->setChecked(true);
	}
	else
	{
		ui.action_RenderWireframe->setChecked(false);
		ui.action_RenderPoints->setChecked(false);
		m_OpenglView.setMode(GL_FILL);
		m_OpenglView.updateGL();
	}
}
// Display about GLC_Player dialog box
void glc_player::aboutPlayer()
{
	AboutPlayer about(this);
	about.exec();
}

// Go to fullScreen Mode
void glc_player::fullScreen()
{
	// Toggle fullScreen Mode
	setWindowState(windowState() ^ Qt::WindowFullScreen);	
}
// Refresh the icon in item list
void glc_player::refreshIconInItemList()
{
	if ((NULL != m_pCurrentSelection))
	{
		unselect();
		m_OpenglView.unselect();
	}	
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	// The number of item in the list
	const int max= ui.loadingList->count();
	// An iterator to the FileEntry HashTable
	QHash<QString, FileEntry>::iterator iEntry;	
	if (m_DisplayThumbnails)
	{
		// Thumbnail have to be displayed
		ui.loadingList->setIconSize(m_iconSize);
		
		// Show the progress bar
		ui.statusbar->showMessage(tr("Creating Thumbnails Please Wait..."));
		m_pProgressBar->show();
		m_pProgressBar->setValue(0);			
		
		// Get the icon name
		QString iconName= getIconName();
		for (int curListItem= 0; curListItem < max; ++curListItem)
		{
			iEntry= m_FileEntryHash.find(ui.loadingList->item(curListItem)->data(Qt::UserRole).toString());
			if (iEntry.value().isLoaded())
			{
				computeIconInBackBuffer(curListItem);
			}
			else
			{				
				ui.loadingList->item(curListItem)->setIcon(QPixmap(iconName).scaled(m_iconSize, Qt::IgnoreAspectRatio));
			}
			m_pProgressBar->setValue(static_cast<int>(static_cast<double>((curListItem + 1)) * (100.0 / max)));
		}
		m_pProgressBar->hide();
		ui.statusbar->showMessage(tr("Thumbnails successfully created"));
		// Force items to be laid out again
		ui.loadingList->setSpacing(1);
	}
	else
	{
		// Thumbnail haven't to be displayed use the default one
		ui.loadingList->setIconSize(QSize());
		for (int curListItem= 0; curListItem < max; ++curListItem)
		{
			ui.loadingList->item(curListItem)->setIcon(QIcon());
		}
		m_OpenglView.unselect();
	}
	QApplication::restoreOverrideCursor();
	
}
// Display GLC_Player help page
void glc_player::help()
{
	QSettings settings;
	if ("fr" == settings.value("currentLanguage").toString())
	{
		QDesktopServices::openUrl(QUrl("http://glc-player.net/fr/doc.php"));
	}
	else
	{
		QDesktopServices::openUrl(QUrl("http://glc-player.net/doc.php"));
	}
}

//////////////////////////////////////////////////////////////////////
// Private services Functions
//////////////////////////////////////////////////////////////////////

// Create RecentFileActionArray
void glc_player::createRecentFileActionsArray()
{
	QAction* actionExit= ui.menu_File->actions().last();
	
	for (int i= 0; i < MaxRecentFiles; ++i)
	{
		m_pRecentFileActionsArray[i]= new QAction(this);
		m_pRecentFileActionsArray[i]->setVisible(false);
		ui.menu_File->insertAction(actionExit, m_pRecentFileActionsArray[i]);
		connect(m_pRecentFileActionsArray[i], SIGNAL(triggered()), this, SLOT(openRecentFile()));
	}
	m_pRecentFileSeparator= ui.menu_File->insertSeparator(actionExit);
}


// Update Recent Files
void glc_player::updateRecentsFiles()
{
	QMutableStringListIterator i(m_RecentFilesList);
	while (i.hasNext())
	{
		if (!QFile::exists(i.next()))
		{
			i.remove();
		}
	}
	for (int j= 0; j < MaxRecentFiles;  ++j)
	{
		if (j < m_RecentFilesList.count())
		{
			QString text= tr("&%1 %2").arg(j + 1).arg(QFileInfo(m_RecentFilesList[j]).fileName());
			m_pRecentFileActionsArray[j]->setText(text);
			m_pRecentFileActionsArray[j]->setData(m_RecentFilesList[j]);
			m_pRecentFileActionsArray[j]->setVisible(true);
		}
		else
		{
			m_pRecentFileActionsArray[j]->setVisible(false);
		}
	}
}
// Open the file
void glc_player::openFile(const QString& fileName)
{
	QFile file(fileName);
	if (file.exists())
	{
		m_CurrentPath= QFileInfo(fileName).absolutePath();
		m_pProgressBar->setValue(0);
		m_pProgressBar->show();
		m_OpenFileThread.setOpenFile(&file);
		m_OpenFileThread.start(QThread::LowPriority);	
		m_FileLoadingInProgress= true;
		ui.startLoadingButton->setEnabled(false);
		ui.action_NewSession->setEnabled(false);
		ui.refreshThumbnails->setEnabled(false);
		ui.removeUnloadItemButton->setEnabled(false);
		if (m_NumberOfUnloadedFile > 1)
		{
			ui.stopLoadingButton->setEnabled(true);
		}
		else
		{
			ui.stopLoadingButton->setEnabled(false);
		}		
	}
}

// Set the current File
void glc_player::addToRecentFiles(const QString& fileName)
{
	m_CurrentFileName= fileName;
	QString shownName(tr("Untilled"));
	if (!m_CurrentFileName.isEmpty())
	{
		shownName= QFileInfo(m_CurrentFileName).fileName();
		m_RecentFilesList.removeAll(m_CurrentFileName);
		m_RecentFilesList.prepend(m_CurrentFileName);
		if (m_RecentFilesList.size() > MaxRecentFiles)
		{
			while (m_RecentFilesList.size() > MaxRecentFiles)
			{
				m_RecentFilesList.removeLast();
			}
		}
		updateRecentsFiles();
	}
}
// drag enter event
void glc_player::dragEnterEvent(QDragEnterEvent* event)
{
	if (event->mimeData()->hasFormat("text/uri-list"))
	{
		event->acceptProposedAction();
	}
}

// drop event
void glc_player::dropEvent(QDropEvent* event)
{
	QList<QUrl> urls= event->mimeData()->urls();
	if (urls.isEmpty()) return;
	// Stransform the url List to string list
	QStringList localFiles;
	for (int i= 0; i < urls.size(); ++i)
	{
		localFiles << urls[i].toLocalFile();
	}
	if (m_NumberOfUnloadedFile == 0) m_MakeFirstFileCurrent= true;
	addItems(localFiles);
	startLoading();
}

// Load successful
void glc_player::loadSuccessful(QString fileName)
{
	addToRecentFiles(fileName);
	m_FileLoadingInProgress= false;
}

// Add items
void glc_player::addItems(QStringList nameList)
{
	QString currentString;
	QString iconName= getIconName();
	for (int i= 0; i < nameList.size(); ++i)
	{
		// If the file is already in the list the file is skipped
		if (!m_FileEntryHash.contains(nameList[i]))
		{
			++m_NumberOfUnloadedFile;
			QFileInfo fileInfo(nameList[i]);
			QListWidgetItem* pItem= new QListWidgetItem(fileInfo.fileName());
			pItem->setData(Qt::UserRole, QVariant(nameList[i]));
			pItem->setForeground(QBrush(Qt::gray));
			if (m_DisplayThumbnails)
			{
				pItem->setIcon(QPixmap(iconName).scaled(m_iconSize, Qt::IgnoreAspectRatio));
			}
			m_FileEntryHash.insert(nameList[i], FileEntry(nameList[i]));
			ui.loadingList->addItem(pItem);
		}
	}
	updateUIState();
}
// Start Loading FileItem
void glc_player::startLoading()
{
	// Test if application is loading a file
	if (m_FileLoadingInProgress) return;
	
	int curListItem= 0;
	const int max= ui.loadingList->count();
	
	QHash<QString, FileEntry>::iterator iEntry;
	// Test if the list item is empty
	if (curListItem >= max)
	{
		return;
	}
	iEntry= m_FileEntryHash.find(ui.loadingList->item(curListItem)->data(Qt::UserRole).toString());
	while (curListItem < max && !iEntry.value().isReadyToLoad() && m_ContinuListLoading)
	{
		++curListItem;
		if ((curListItem < max))
		{
			iEntry= m_FileEntryHash.find(ui.loadingList->item(curListItem)->data(Qt::UserRole).toString());
		}
	}
	
	// test if the Entry have been find and continu loading == true
	if ((curListItem < max) && m_ContinuListLoading)
	{
		iEntry.value().setLoadingStatus(true);
		openFile(iEntry.value().getFileName());
		m_ListLoadingInProgress= true;
	}
	else if ((curListItem >= max))
	{
		m_ListLoadingInProgress= false;
		ui.startLoadingButton->setEnabled(false);
		ui.stopLoadingButton->setEnabled(false);
		ui.action_NewSession->setEnabled(true);
		ui.refreshThumbnails->setEnabled(m_DisplayThumbnails);
	}
	else
	{
		ui.startLoadingButton->setEnabled(true);
		ui.stopLoadingButton->setEnabled(false);
		ui.action_NewSession->setEnabled(true);
		ui.refreshThumbnails->setEnabled(m_DisplayThumbnails);
		ui.removeUnloadItemButton->setEnabled(true);
	}		
}
// Set the current File Item
void glc_player::setCurrentFileItem(const QString& fileName)
{
	QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(fileName);
	if ((iEntry != m_FileEntryHash.constEnd()) && (iEntry.value().isLoaded()))
	{
		// Set polygon mode of the entry
		iEntry.value().setPolygonMode(m_OpenglView.getMode());
		GLC_Instance instance(iEntry.value().getInstance());
		m_OpenglView.clear();
		m_OpenglView.add(instance);
		if (iEntry.value().cameraIsSet())
		{
			m_OpenglView.setCamera(iEntry.value().getCamera());
			m_OpenglView.updateGL();
		}
		else
		{
			m_OpenglView.reframe(GLC_BoundingBox(), false);
		}
		setWindowTitle(QString(QCoreApplication::applicationName() +" [") + QFileInfo(fileName).fileName() + QString("]"));
		ui.statusbar->showMessage(fileName);
		ui.fileSize->setText(QString::number(QFileInfo(fileName).size() / 1000));
		ui.numberOfMesh->setText(QString::number(iEntry.value().getNumberOfInstance()));
		ui.numberOfVertex->setText(QString::number(iEntry.value().getNumberOfVertex()));
		ui.numberOfFaces->setText(QString::number(iEntry.value().getNumberOfFaces()));
		// Enable the possibility to remove current item
		m_pActionDeleteSelectedItem->setEnabled(true);		
	}
	else if (iEntry.value().isLoading())
	{
		// Disable the possibility to remove current item
		m_pActionDeleteSelectedItem->setEnabled(false);		
		m_OpenglView.clear();
		m_OpenglView.updateGL();		
	}
	else // Entry not found or not loaded => clear the view
	{
		m_OpenglView.clear();
		m_OpenglView.updateGL();
	}
}
// Update the UI State
void glc_player::updateUIState()
{
	if (ui.loadingList->count() == 0)
	{
		// Disable New session action
		ui.action_NewSession->setEnabled(false);
		// Disable Refresh thumbnail button
		ui.refreshThumbnails->setEnabled(false);
		// Disable all loading list action
		ui.startLoadingButton->setEnabled(false);
		ui.stopLoadingButton->setEnabled(false);
		ui.removeUnloadItemButton->setEnabled(false);
		ui.removeOnErrorItem->setEnabled(false);
		// Uncheck start and stop button
		ui.stopLoadingButton->setChecked(false);
		ui.startLoadingButton->setChecked(false);
		//Update window Title
		setWindowTitle(QCoreApplication::applicationName());
	
		ui.statusbar->showMessage(tr("Untiteled"));
		ui.fileSize->clear();
		ui.numberOfMesh->clear();
		ui.numberOfVertex->clear();
		ui.numberOfFaces->clear();
		// Disable the possibility to remove current item
		m_pActionDeleteSelectedItem->setEnabled(false);		
		
	}else if (0 == m_NumberOfUnloadedFile)
	{
		// Disable action related action
		ui.startLoadingButton->setEnabled(false);
		ui.stopLoadingButton->setEnabled(false);
		ui.removeUnloadItemButton->setEnabled(false);
		// Uncheck start and stop buttons
		ui.stopLoadingButton->setChecked(false);
		ui.startLoadingButton->setChecked(false);
	}
	ui.numberOfItem->setText(QString::number(ui.loadingList->count()));
	ui.numberOfLoadedItem->setText(QString::number(ui.loadingList->count() - m_NumberOfUnloadedFile));
	ui.removeOnErrorItem->setEnabled(m_NumberOfErrorFile != 0);
}

// Compute the icon of a newly loaded file in back buffer
void glc_player::computeIconInBackBuffer(int itemRow)
{
	GLC_Camera savCam;
	bool itemIsNotCurrent= true;
	const int currentRow= ui.loadingList->currentRow();
	bool viewIsEmpty= m_OpenglView.isEmpty();
	m_OpenglView.setSnapShootMode(true);
	// Test if the current item is already in the current view and loaded
	if ((itemRow != currentRow) || (itemRow == currentRow && viewIsEmpty))
	{
		QString fileName(ui.loadingList->item(itemRow)->data(Qt::UserRole).toString());
		
		QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(fileName);
		if (iEntry.value().isLoaded())
		{
			savCam= m_OpenglView.getCamera();
			// Set polygon mode of the entry
			iEntry.value().setPolygonMode(m_OpenglView.getMode());
			iEntry.value().setIconMode(m_OpenglView.getMode());
			GLC_Instance instance(iEntry.value().getInstance());
			m_OpenglView.setAutoBufferSwap(false);
			m_OpenglView.clear();
			m_OpenglView.add(instance);
			if (iEntry.value().cameraIsSet())
			{
				m_OpenglView.setCamera(iEntry.value().getCamera());
				m_OpenglView.updateGL();
			}
			else
			{
				GLC_BoundingBox emptyBoundingBox;
				m_OpenglView.isoView(false);
				m_OpenglView.reframe(emptyBoundingBox, false);
				iEntry.value().setCamera(m_OpenglView.getCamera());
			}			
		}
		
	}
	else
	{
		m_OpenglView.setAutoBufferSwap(false);
		m_OpenglView.updateGL();
		itemIsNotCurrent= false;
		m_OpenglView.setAutoBufferSwap(true);
	}
	// Make the screenShoot
	QImage snapShoot= m_OpenglView.grabFrameBuffer();
	ui.loadingList->item(itemRow)->setIcon(QPixmap::fromImage(snapShoot).scaled(m_iconSize, Qt::IgnoreAspectRatio));
	
	if (itemIsNotCurrent)
	{
		QString fileName(ui.loadingList->currentItem()->data(Qt::UserRole).toString());
		
		QHash<QString, FileEntry>::iterator iEntry= m_FileEntryHash.find(fileName);
		if (iEntry.value().isLoaded())
		{
			// Set polygon mode of the entry
			iEntry.value().setPolygonMode(m_OpenglView.getMode());
			GLC_Instance instance(iEntry.value().getInstance());
			m_OpenglView.clear();
			m_OpenglView.add(instance);
			if (viewIsEmpty)
			{
				m_OpenglView.setAutoBufferSwap(true);
				m_OpenglView.reframe(GLC_BoundingBox(), false);
				iEntry.value().setCamera(m_OpenglView.getCamera());
			}
			else
			{
				m_OpenglView.setCamera(savCam);
				m_OpenglView.updateGL();
				m_OpenglView.setAutoBufferSwap(true);				
			}
			

		}
		else
		{
			m_OpenglView.clear();
			m_OpenglView.setAutoBufferSwap(true);
			m_OpenglView.updateGL();
		}
	}
	
	m_OpenglView.setSnapShootMode(false);
}

// Return icon name with a size
QString glc_player::getIconName() const
{
	// Built the icon name
	QString iconName(":/images/Logo");
	if (m_iconSize.width() == 32)
	{
		iconName.append(".png");
	}
	else if (m_iconSize.width() == 40)
	{
		iconName.append("40x40.png");
	}	
	else if (m_iconSize.width() == 50)
	{
		iconName.append("50x50.png");
	}
	else if (m_iconSize.width() == 60)
	{
		iconName.append("60x60.png");
	}
	else if (m_iconSize.width() == 70)
	{
		iconName.append("70x70.png");
	}	
	else if (m_iconSize.width() == 80)
	{
		iconName.append("80x80.png");
	}
	return iconName;
}

// Read Application Settings
void glc_player::readSettings()
{
	QSettings settings;
	// Recent File list
	m_RecentFilesList= settings.value("recentFiles").toStringList();	
	
	// MainWindow geometry
	settings.beginGroup("MainWindow");
	resize(settings.value("size", QSize(890, 520)).toSize());
	move(settings.value("pos", QPoint(50, 50)).toPoint());
	m_dislayInfoPanel= settings.value("displayInfoPanel", true).toBool();
	settings.endGroup();
	
	// Session Manager
	settings.beginGroup("SessionManager");
	// Session manager visibility
	bool visibility= settings.value("visibility", true).toBool();
	ui.action_SessionManagement->setChecked(visibility);
	ui.sessionManagementWindow->setVisible(visibility);
	// Item Information visibility
	visibility= settings.value("itemInformationVisibility", false).toBool();
	ui.displayItemCheckBox->setChecked(visibility);
	ui.currentItemGroup->setVisible(visibility);
	// IconSize
	m_iconSize= settings.value("thumbnailSize", QSize(40, 40)).toSize();
	// Thumbnails display
	m_DisplayThumbnails= settings.value("displayThumbnail", true).toBool();
	// Thumbnails creation
	m_CreateThumbnailWhileLoading= settings.value("createThumbnail", true).toBool();
	settings.endGroup();
}

// Write Application Settings
void glc_player::writeSettings()
{
	QSettings settings;
	
	// Recent File list
	settings.setValue("recentFiles", m_RecentFilesList);
	
	// MainWindow geometry
	if (isFullScreen())
	{
		// Toggle fullScreen Mode
		setWindowState(windowState() ^ Qt::WindowFullScreen);			
	}
	// MainWindow geometry
	settings.beginGroup("MainWindow");	
	settings.setValue("size", size());
	settings.setValue("pos", pos());
	settings.setValue("displayInfoPanel", m_dislayInfoPanel);
	settings.endGroup();

	// Session Manager
	settings.beginGroup("SessionManager");
	// Session manager visibility
	settings.setValue("visibility", ui.sessionManagementWindow->isVisible());
	// Item Information visibility
	settings.setValue("itemInformationVisibility", ui.currentItemGroup->isVisible());
	// IconSize
	settings.setValue("thumbnailSize", m_iconSize);
	// Thumbnails display
	settings.setValue("displayThumbnail", m_DisplayThumbnails);
	// Thumbnails creation
	settings.setValue("createThumbnail", m_CreateThumbnailWhileLoading);
	settings.endGroup();
}


